#! /usr/bin/env python

### Interactive (dialog based) configuration
### Allows multi-monitor setups, selecting a primary video card in the case of
### multiple screens and other interesting possibilities

try:
	import sys, os, commands
	from dialog import *
	from mouse import XAC_Mouse, get_mouse
	from keyboard import XAC_Keyboard, get_kbd_models, get_kbd_layouts, get_kbd_variant, get_kbd_options
	from monitor import XAC_Monitor

except ImportError, e:
	print "Module loading failed with: " + str(e)
	print "Please check that xac was installed correctly"

### Note that in all of the below, c == XAC_Config reference

###############################
### Create the main screen menu
###############################
def main_menu(c):
	mm = Dialog_menu('XAC v' + c.version + ' - Main Menu')
	### Configure Video Card(s)
	mm.add('Video Cards', 'Configure Video Card(s)')
	### Configure the input devices
	mm.add('Input', 'Configure input devices')
	### Other Server Settings
	mm.add('Server', 'Server Options')
	### Save configuration
	mm.add('Create Config', 'Save your changes and create the configuration file')
	### Exit
	mm.add('Quit', 'Quit without saving')
	return mm

#################################
### Create the save and exit menu
#################################
def save_exit(c):
	se = Dialog_menu('Save and Exit')
	### Filename to write to
	se.add('Output file:', c.filename)
	### Save
	se.add('Save', 'Save to ' + c.filename + ' and quit')
	### Return back to main menu
	se.add('Cancel', 'Return to main menu')
	### Exit (Quit without saving)
	se.add('Quit', 'Quit without saving')
	return se

#########################
### Configuration Summary
#########################
def configuration_summary(c):
	cs = Dialog_msgbox("WARNING: Interactive Mode is not yet complete!\nUse at your own risk!\nAutodetection Configuration Summary").draw()

#############################
### Configures the video card
#############################
### Selected Card
def config_video_card(c, id):
	cards = c.arch.cards
	cv = Dialog_menu("Configure Video Card")
	### Selected Card (Show PCI ID + PCI String for PCI
	if cards[id].pci_loc:
		cv.add('Selected Card:',  cards[id].name + ' - (' + cards[id].get_pci_loc() + ')')
	### Selected Card (Show FB + FB ID String for FB)
	else:
		cv.add('Selected Card:', '(Unknown) - ' + cards[id].name) 
	### Override driver? (not recommended)
	cv.add('Selected Driver:', cards[id].driver.driver_description)
	### Card is multihead?
	if cards[id].driver.multihead:
		if cards[id].driver.multihead_type == "None":
			cv.add('Multihead:', 'Off')
		else:
			cv.add('Multihead:', 'On (' + cards[id].driver.multihead_type + ')')
	### Does the card support DRI?
	if cards[id].driver.dri:
		### Config allows DRI?
		if c.dri:
			cv.add('DRI:', 'Enabled')
		else:
			cv.add('DRI:', 'Disabled')
	### Monitor(s) attached to this card
	cv.add('Monitors', 'Configure monitors connected to this card')
	### Go Back
	cv.add('Back', 'Go back to the main menu')
	return cv

### Choose between video cards available
def select_video_card(c, id):
	sv = Dialog_menu("Select Video Card (the current card has a *)")
	count = 1
	for c in c.arch.cards:
		if count == id + 1:
			name = "* " + c.name
		else:
			name = c.device_name

		if c.pci_loc:
			sv.add(str(count), name + ' - (' + c.get_pci_loc() + ')')
		else:
			sv.add(str(count), name + ' - (Unknown)')

		count = count + 1
	return sv

### Choose between drivers available, pass 1 for all_drivers to see *all drivers*
def select_video_driver(c, id, all_drivers=0):
	if all_drivers:
		sv = Dialog_menu("All available drivers: (the current driver has a *)")
		pass
	else:
		sv = Dialog_menu("Detected compatible drivers for this video card, (the current driver has a *)")
		for d in c.arch.cards[id].compatible_drivers:
			if c.arch.cards[id].driver.driver == d.driver:
				sv.add("* " + d.driver, d.driver_description)
			else:
				sv.add(d.driver, d.driver_description)
	return sv

#####################
### Configure Monitor
#####################
### Add a monitor
def add_monitor(c, id):
	card = c.arch.cards[id]
	m = XAC_Monitor(id = len(c.arch.cards[id].monitors))	
	name = Dialog_inputbox("Please enter a name for this monitor").draw()
	if name == 0 or name == '':
		return
	else:
		m.name = name
	### Ask for the Horizontal Range
	h_min = Dialog_inputbox("Please give the horizontal sync minimum frequency:").draw()
	h_max = Dialog_inputbox("Please give the horizontal sync maximum frequency:").draw()
	if h_min and h_max:
		m.sync['h_min'] = h_min
		m.sync['h_max'] = h_max
	else:
		return

	### Ask for the Vertical Range
	v_min = Dialog_inputbox("Please give the vertical sync minimum frequency:").draw()
	v_max = Dialog_inputbox("Please give the vertical sync maximum frequency:").draw()
	if v_min and v_max:
		m.sync['v_min'] = v_min
		m.sync['v_max'] = v_max
	else:
		return

	### Ask for the Supported Screen Resolutions
	### Ask for the Supported Depths
	### Set the monitor position
	m.position = monitor_placement(c, id, m.id).draw()
	### If we're good to add the monitor, add it
	card.monitors.append(m)

### Select a monitor to configure (pass in the card id that this is attached to)
def select_monitor(c, id):
	card = c.arch.cards[id]
	cm = Dialog_menu("Configure monitors attached to " + card.name)
	for m in card.monitors:
		print m.monitor_name()
		cm.add('Monitor' + str(m.id), m.monitor_name())
	cm.add('Add', 'Add a montitor to this Video Card')
	cm.add('Back', 'Back to the Video Card Configuration')
	return cm

### Move Monitor to a different card
def move_monitor(c, id, mid):
	m = c.arch.cards[id].monitors[mid]
	mm = Dialog_menu('Move Monitor: ' + m.name + ' to a different video card')
	card = c.arch.cards[id]
	for card in c.arch.cards:
		if card == c.arch.cards[id]:
			mm.add('Card' + str(card.id), '* ' + card.name)		
		else:
			mm.add('Card' + str(card.id), card.name)
	mm.add('Back', 'Back to the Monitor Configuration Menu')
	return mm

### Monitor Placement
def monitor_placement(c, id, mid):
	matrix =          "If this screen is the main screen, press 5.  Otherwise, indicate this screen's position in relation to the main screen at position 5, using the chart below.\n" 
	matrix = matrix + "\n"
	matrix = matrix + "        Above\n"
	matrix = matrix + "  +---+ +---+ +---+\n"
	matrix = matrix + "  | 1 | | 2 | | 3 |\n"
	matrix = matrix + "L +---+ +---+ +---+ R \n"
	matrix = matrix + "e +---+ +---+ +---+ i\n"
	matrix = matrix + "f | 4 | | 5 | | 6 | g\n"
	matrix = matrix + "t +---+ +---+ +---+ h\n"
	matrix = matrix + "  | 7 | | 8 | | 9 | t\n"
	matrix = matrix + "  +---+ +---+ +---+\n"
	matrix = matrix + "        Below\n"
	mp = Dialog_inputbox(matrix)
	return mp

### Monitor Configuration (pass in the card id and "monitor id" (mid) this is attached to)
def configure_monitor(c, id, mid):
	m = c.arch.cards[id].monitors[mid]
	cm = Dialog_menu("Monitor " + str(mid) +  ": " + m.monitor_name())
	### Resolution
	#cm.add("Display Resolutions:", )
	### Display Depth
	#cm.add("Display Depths:", )
	### Monitor Vertical Range
	cm.add('Vertical Range', str(m.sync['v_min']) + '-' + str(m.sync['v_max']))
	### Monitor Horizontal Range
	cm.add('Horizontal Range', str(m.sync['h_min']) + '-' + str(m.sync['h_max']))
	### Move monitor to a different card
	cm.add("Move Monitor", "Move monitor to a different card")
	cm.add('Back', 'Back to the monitor selection screen')
	return cm

#######################
### Input Configuration
#######################
### Input configuration
def input_config(c):
	ic = Dialog_menu("Input Device Configuration")
	for k in c.kbds:
		ic.add('Keyboard' + str(k.id), k.name)
	for m in c.mice:
		ic.add('Mouse' + str(m.id), m.name)
	ic.add('Add', 'Add an input device')
	ic.add('Back', 'Go back to the main menu')
	return ic

### Keyboard configuration
def keyboard_config(c, kbd):
	kc = Dialog_menu('Keyboard' + str(kbd) + ' Configuration')
	kc.add('Model:', c.kbds[kbd].model)
	kc.add('Layout:', c.kbds[kbd].layout)
	if len(c.kbds[kbd].variant) == 0:
		kc.add('Variant:', "None")
	else:
		kc.add('Variant:', c.kbds[kbd].variant)
	kc.add('Delete', 'Delete this device')
	kc.add('Back', 'Go back to the main menu')
	return kc

### Switch keyboard model
def keyboard_model():
	km = Dialog_menu("Select Keyboard Model - Unless you're sure that you need a different model, please use the generic pc10x model")
	for k in get_kbd_models():
		km.add(k[0], k[1])
	return km

### Switch keyboard layout
def keyboard_layout():
	kl = Dialog_menu("Select Keyboard Layout")
	for k in get_kbd_layouts():
		kl.add(k[0], k[1])
	return kl

### Switch keyboard variant
def keyboard_variant(c, kbd):
	kv = Dialog_menu("Select Keyboard Variant")
	for k in get_kbd_variant(c.kbds[kbd].layout):
		kv.add(k[0], k[1])
	return kv

### Mouse configuration
def mouse_config(c, mouse):
	mc = Dialog_menu('Mouse' + str(mouse) + ' Configuration')
	mc.add('Driver:', c.mice[mouse].driver)
	mc.add('Device:', c.mice[mouse].device)
	### Add option for finger or corner tap on synaptics
	if c.mice[mouse].driver == "synaptics":
		mc.add( 'Tap Buttons:', c.mice[mouse].tapping) 
	mc.add('Delete', 'Delete this device')
	mc.add('Back', 'Go back to the main menu')
	return mc

### Mouse Driver Switcher
def mouse_driver(c, mouse):
	md = Dialog_menu('Available Mouse Drivers')
	md.add('mouse', 'Generic Mouse')
	md.add('serial', 'Serial Mouse')
	md.add('synaptics', 'Synaptics/ALPS Touchpad')	
	md.add('Back', 'Go back to the mouse configuration')
	return md

### Tap Buttons Configuration
def tap_buttons(c, mouse):
	tb = Dialog_menu('Tap Buttons Configuration for Mouse' + str(mouse))
	tb.add('None', 'Do not accept taps as button presses')
	tb.add('Single Tap', 'Only allow taps for left button presses')	
	tb.add('Finger Tap', 'Finger taps correspond to button presses')
	tb.add('Corner Tap', 'Corner taps correspond to button presses')
	return tb

### Add an input device
def add_input_dev(c):
	aid = Dialog_menu('Add Input Device')
	aid.add('Mouse', 'Add a mouse-like input device')
	aid.add('Keyboard', 'Add a keyboard-like input device')
	return aid

### Other Server Settings
def other_server_settings(c):
	oss = Dialog_menu('Server Settings')
	oss.add('Configure Power Saving Settings','')
	### Fail on mouse failure?
	if c.mouse_fail:
		oss.add('Allow the server to start without a mouse', 'True')
	else:
		oss.add('Allow the server to start without a mouse', 'False')
	### Allow mode change?
	if c.allow_mode_change:
		oss.add('Prevent the server from switching display resolutions', 'False')
	else:
		oss.add('Prevent the server from switching display resolutions', 'True')

	oss.add('Back', 'Go back to the main menu')
	return oss

### Interact with the user
def interact(c):
	### Give initial detected configuration
	configuration_summary(c)

	### Okay, the user is informed, let's interact
	mm = main_menu(c)
	while(1):
		result = mm.draw()
		
		### Quit
		if result == "Quit" or result == 0:
			sys.exit()
		### Video Card menu
		elif result == "Video Cards":
			id = 0
			while(1):
				result = config_video_card(c, id).draw()
				if result == 'Back' or result == 0:
					break
				elif result == "Selected Card:":
					result = int(select_video_card(c, id).draw())
					if result:
						id = result - 1
				elif result == "Selected Driver:":
					result = select_video_driver(c, id).draw()
					for d in c.arch.cards[id].compatible_drivers:
						if result == d.driver:
							c.arch.cards[id].driver = d
				elif result == "Multihead:":
					d = c.arch.cards[id].driver
					mlen = len(d.multihead_options)
					for i in range(0, mlen):
						if d.multihead_options[i] == d.multihead_type:
							if i + 1 >= mlen:
								d.multihead_type = d.multihead_options[0]
							else:
								d.multihead_type = d.multihead_options[i + 1]
							break
				elif result == "DRI:":
					if c.dri:
						c.dri = 0
					else:
						c.dri = 1	
				elif result == "Monitors":
					while(1):
						result = select_monitor(c, id).draw()
						if result == 'Back' or result == 0:
							break
						### Add a monitor to the config
						elif result == 'Add':
							add_monitor(c, id)
						### It's a monitor
						else:
							mid = int(result[7:])
							cm = configure_monitor(c, id, mid)		
							while(1):
								result = cm.draw()
								if result == 'Back' or result == 0:
									break
								### Ask for the Horizontal Range
								elif result == 'Vertical Range':
									m = c.arch.cards[id].monitors[mid]
									v_min = Dialog_inputbox("Please give the vertical sync minimum frequency:").draw()
									v_max = Dialog_inputbox("Please give the vertical sync maximum frequency:").draw()
									if v_min and v_max:
										m.sync['v_min'] = v_min
										m.sync['v_max'] = v_max
									print m.sync
								### Ask for the Vertical Range
								elif result == 'Horizontal Range':
									m = c.arch.cards[id].monitors[mid]
									h_min = Dialog_inputbox("Please give the horizontal sync minimum frequency:").draw()
									h_max = Dialog_inputbox("Please give the horizontal sync maximum frequency:").draw()
									if h_min and h_max:
										m.sync['h_min'] = h_min
										m.sync['h_max'] = h_max
									print m.sync
								### Move monitor to a different card
								elif result == 'Move Monitor':
									nc = move_monitor(c, id, mid).draw()
									if nc == 'Back' or nc == 0:
										pass
									else:
										nc = int(nc[4:])
										if not nc == id:
											c.arch.cards[nc].monitors.append(c.arch.cards[nc].monitors[mid])
											c.arch.cards[nc].monitors[mid].delete()
		### Input menu
		elif result == "Input":
			while(1):
				result = input_config(c).draw()
				if result == 'Back' or result == 0:
					break
				elif result[:8] == "Keyboard":
					kbd = int(result[8])
					while(1):
						result = keyboard_config(c, kbd).draw()
						if result == 'Back' or result == 0:
							break
						elif result == 'Model:':
							result = keyboard_model().draw()	
							if result == 'Back' or result == 0:
								pass
							else:
								c.kbds[kbd].model = result
						elif result == 'Layout:':
							result = keyboard_layout().draw()	
							if result == 'Back' or result == 0:
								pass
							else:
								c.kbds[kbd].layout = result
						elif result == 'Variant:':
							result = keyboard_variant(c, kbd).draw()
							if result == 'Back' or result == 0:
								pass
							else:
								c.kbds[kbd].variant = result
						elif result == 'Options':
							pass
						elif result == 'Delete':
							if Dialog_yesno("Are you sure you want to delete this keyboard?").draw() != 0:	
								### Delete it	
								del c.kbds[kbd]
								break
							
				elif result[:5] == "Mouse":
					mouse = int(result[5])
					while(1):
						result = mouse_config(c, mouse).draw()
						if result == 'Back' or result == 0:
							break
						elif result == 'Driver:':
							result = mouse_driver(c, mouse).draw()
							if result == 'Back' or result == 0:
								pass
							else:
								c.mice[mouse] = get_mouse(result, c.mice[mouse].device, id=mouse, machine=c.mice[mouse].machine)
						elif result == 'Device:':
							dev = Dialog_inputbox("Device path")
							### FIXME Check the device path
							dev.add(c.mice[mouse].device)	
							c.mice[mouse].device = dev.draw()
						elif result == 'Tap Buttons:':
							tap = tap_buttons(c, mouse).draw()
							if tap:
								c.mice[mouse].tapping = tap
						elif result == 'Delete':
							if Dialog_yesno("Are you sure you want to delete this mouse?").draw() != 0:	
								### Delete it	
								del c.mice[mouse]
								break
				elif result == "Add":
					### Ask, Mouse or Keyboard?
					result = add_input_dev(c).draw()
					if result == 'Back' or result == 0:
						### Don't break, we're not in a loop
						pass 
					elif result == "Mouse":
						c.mice.append(XAC_Mouse(None, len(c.mice), None, c.arch.get_machine()))	
					elif result == "Keyboard":
						new_id = len(c.kbds)
						c.kbds.append(XAC_Keyboard(len(c.kbds)))
		elif result == "Server":
			while(1):
				result = other_server_settings(c).draw()
				if result == 'Back' or result == 0:
					break;
				elif result.count("Allow"):
					if c.mouse_fail:
						c.mouse_fail = 0
					else:
						c.mouse_fail = 1
				elif result.count("Prevent"):
					if c.allow_mode_change:
						c.allow_mode_change = 0
					else:
						c.allow_mode_change = 1
		elif result == "Create Config":
			### Sanity checks before creating a config
			if len(c.mice) == 0 and not c.mouse_fail:
				Dialog_msgbox("Please configure a mouse first!").draw()
			elif len(c.kbds) == 0:
				Dialog_msgbox("Please configure a keyboard first!").draw()	
			elif len(c.arch.cards) == 0:
				Dialog_msgbox("Please configure a Video Device first!").draw()
			### Add more?
			else:
				while(1):
					result = save_exit(c).draw()
					if result == 'Cancel' or result == 0:
						break
					elif result == "Output file:":
						fn = Dialog_inputbox("Output file")
						if c.filename:
							fn.add(c.filename)
							filename = fn.draw()
							if filename:
								c.filename = filename
					elif result == "Save":
						c.save_config()		
						Dialog_msgbox('Saved configuration to:\n' + c.filename).draw()
						sys.exit()
					elif result == "Quit":
						sys.exit()
