#! /usr/bin/env python

from template import Section

lst_dir = '/usr/share/X11/xkb/rules/xorg.lst'
kbd_models = []		### List of model / description pairs
kbd_layouts = []	### Layout / description
kbd_variant = {}	### Layout -> ( Variant / description )
kbd_options = {}	### Option -> ( Variant / description )

### Try to parse the xorg.lst
try:
	lst = open(lst_dir, mode="r")
except:
	print "### Unable to read " + lst_dir
	sys.exit(1)

active = kbd_models
for line in lst.readlines():
	if line.count("! model"):
		active = kbd_models
	elif line.count("! layout"):
		active = kbd_layouts
	elif line.count("! variant"):
		active = kbd_variant
	elif line.count("! option"):
		active = kbd_options
	### Parse each info line
	else:
		### Workaround for the only line that doesn't work :p		
		line.replace("Compose key", "compose")
		### Split it
		unpack = line[2:].replace("\n","").split(" ",1)
		### Skip empty lines
		if len(unpack) == 1:
			continue
		unpack[1] = unpack[1].strip()
		if active == kbd_models or active == kbd_layouts:
			active.append(unpack)
		elif active == kbd_variant:
			secondary = unpack[1].split(":",1)
			secondary[1] = secondary[1].strip()
			### Add the key if needed
			value = [unpack[0], secondary[1]]
			key = secondary[0]	
			if active.has_key(key):	
				active[key].append(value)
			else:
				active[key] = [value]
		elif active == kbd_options:
			pass

### Get lists
def get_kbd_models():
	return kbd_models
def get_kbd_layouts():
	return kbd_layouts
def get_kbd_variant(layout):
	for k in kbd_variant:
		if k == layout:
			return kbd_variant[k]
	return []
def get_kbd_options():
	return kbd_options

### Sets up the keyboard
class XAC_Keyboard:
	def __init__(self, id=0):
		### Device #
		self.id = id
		self.layout = "us"
		self.model = "pc105"
		self.name = "Generic Keyboard"
		self.variant = []

	def get_section(self):
		section = Section()
		section.name('InputDevice')
		section.add('Identifier\t"Keyboard' + str(self.id) + '"')
		section.add('Driver\t\t"kbd"')
		section.add('Check /usr/share/X11/xkb/rules/xorg.lst for available models',3)
		section.add('Option\t\t"XkbdModel"\t\t"' + self.model +'"')
		section.add('Check /usr/share/X11/xkb/symbols/ for available layouts',3)
		section.add('Option\t\t"XkbLayout"\t\t"' + self.layout + '"')
		section.add('Option\t\t"XkbOptions"\t\t"compose:rwin"')
		if len(self.variant):
			section.add('Uncomment the statement below if your layout uses nodeadkeys', 3)
			section.add('Option\t"XkbVariant"\t\t"nodeadkeys"', 1)
			section.add('Uncomment the statement below if you use a dvorak layout', 3)
			section.add('Option\t"XkbVariant"\t\t"dvorak"', 1)
		else:
			for v in self.variant:
				section.add('Option\t"XkbVariant"\t\t"' + v + '"')
		return section.get_section()
