#! /usr/bin/env python

from template import Section
import commands
import os

### Use cvt or gtf?
use_cvt = 0
if os.path.isfile('/usr/bin/cvt'):
	use_cvt = 1

### Some assorted functions
def get_cvt_mode(x, y, reduced = 0):
	if reduced:
		return commands.getoutput('cvt -r ' + str(x) + ' ' + str(y))
	else:
		return commands.getoutput('cvt ' + str(x) + ' ' + str(y))

def get_gtf_mode(x, y):
	return commands.getoutput('gtf ' + str(x) + ' ' + str(y) + ' 60').strip()

def get_mode(x,y, reduced = 0):
	if use_cvt:
		result = get_cvt_mode(x, y, reduced)
	else:
		if reduced:
			print "Warning: reduced modes aren't found with gtf."
		result = get_gtf_mode(x,y)
	
	split = result.find('\n')
	comment = result[:split]
	line = result[split + 1:].split(" ")
	line[1] = '"%dx%d"' % (x, y)
	return (comment, " ".join(line))

def reduced_blanking(x, y):
	mode = get_mode(x, y)

	p_loc = 0
	p_clk = 0

	for p in mode[0].split():
		if p.count('pclk'):
			p_clk = mode[0].split()[p_loc + 1]
		else:
			p_loc = p_loc + 1
	
	if eval(p_clk) >= 165:
		return 1
	else:
		return 0

### Defines a modeline
class Modeline:
	### If only xres and yres are given, we generate the modeline
	def __init__(self, xres, yres, comment = None, line = None, rb = 0):
		self.xres = xres
		self.yres = yres
		self.reduced_blanking = rb
		self.comment = ""
		self.line = ""

		if not comment or not line:
			if reduced_blanking(xres, yres):
				self.reduced_blanking = 1
				(self.comment, self.line) = get_mode(xres,yres, 1)
			else:
				(self.comment, self.line) = get_mode(xres,yres, 0)
		else:
			self.comment = comment
			self.line = line

	### s is a Section reference
	def get_modeline(self, s):
		s.add(self.comment, 3)
		s.add(self.line)

### Defines a mode
class Mode:
	def __init__(self, horiz = 0, vert = 0, depth = 0, modeline = None):
		self.horiz = horiz
		self.vert = vert
		self.depth = depth
		if modeline:
			self.modeline = modeline
		else:
			self.modeline = Modeline(horiz, vert)
	
	### Check if one mode is equal to another
	def __cmp__(self, mode):
		if self.horiz == mode.horiz and self.vert == mode.vert and self.depth == mode.depth:
			### Return 0 on equal
			return 0
		else:
			### I'm not going to implement < >, etc., so
			### return anything but 0
			return 1

	### Resolution string
	def res(self):
		return '"' + str(self.horiz) + "x" + str(self.vert) + '"'
	
	### Prints the modeline for this mode
	def get_modeline(self, s):
		if not is_vesa_Mode(self):
			ml = self.modeline.get_modeline(s)

### List of VESA modes (no text modes or modes < 8bit)
### We shouldn't need modelines for VESA modes
VESA_MODES = [
	# VESA 1.0 / 1.1 (no text modes or modes < 8bit)
	Mode(640, 400, 8), Mode(640, 480, 8), Mode(800, 600, 8),
	Mode(1024,768, 8), Mode(1280, 1024, 8),

	# VESA 1.2+
	Mode(320, 200, 15), Mode(320, 200, 16), Mode(320, 200, 24),
	Mode(640, 480, 15), Mode(640, 480, 16), Mode(640, 480, 24),
	Mode(800, 600, 15), Mode(800, 600, 16), Mode(800, 600, 24),
	Mode(1024, 768, 15), Mode(1024, 768, 16), Mode(1024, 768, 24),
	Mode(1280, 1024, 15), Mode(1280, 1024, 16), Mode(1280, 1024, 24),

	# VESA 2.0+
	Mode(1600, 1200, 8), Mode(1600, 1200, 15), Mode(1600, 1200, 16)
]

def is_vesa_Mode(mode):
	for m in VESA_MODES:
		if mode == m:
			return 1
	return 0

### Pass in an edid and/or an fb reference to setup the monitor
### If neither is passed in, we assume it's a monitor capable of
### 640x480-8@60
class XAC_Monitor:
	def __init__(self, edid = None, fb = None, id=0, default_depth = None):
		### Monitor Name
		self.name = "Unknown Monitor"
		### Forced default depth
		self.default_depth = default_depth
		### Display depths
		self.depth = [8, 16, 24]
		### FB Depth
		self.fbdepth = 0
		### Valid modes (default to the *first* one)
		self.modes = []
		### ID # for the monitor
		self.id = id
		### Sync values
		self.sync = {}
		self.sync['h_min'] = 0
		self.sync['h_max'] = 0
		self.sync['v_min'] = 0
		self.sync['v_max'] = 0
		### Position, 5 is the main monitor
		self.position = 5
		
		### Monitor's EDID
		if edid and edid.valid:
			self.edid = edid
		else:
			self.edid = None
		### Monitor's FB Settings
		if fb and fb.valid:
			self.fb = fb
		else:
			self.fb = None
		
		### Use the FB settings to setup the monitor
		if self.fb != None:
			var = self.fb.var
			
			### Add depth if it's not already there
			### Covers 32bit and 15 bit displays
			if not self.depth.count(self.fb.depth()):
				self.depth.append(self.fb.depth())
			self.fbdepth = self.fb.depth()

			res = self.fb.res()
			
			### If it's 640 by 480, let's bump up to 800x600,
			### probably a safe bet...
			if res[0] == 640 and res[1] == 480:
				res[0] = 800
				res[1] = 600

			self.modes = [ self.fb.get_mode() ] + self.modes

		### Use the EDID to setup the monitor
		elif edid != None:
			details = edid.get_monitor_details()
			for d in details:
				if d[0].count("Range") and len(d) == 2:
					r = d[1]
					self.sync['h_min'] = r['h_min']
					self.sync['h_max'] = r['h_max']
					self.sync['v_min'] = r['v_min']
					self.sync['v_max'] = r['v_max']

			### Workaround for some Apple LCD panels
			id = edid.get_id_string()
			if ["APP1592", "APPF401", "APP1792", "APP1992", "APP1692", "APP1892"].count(id):
				self.sync['v_min'] = 50
				self.sync['v_max'] = 60
			
			### We default to the first detailed timing
			### Hopefully, it's the monitor's native resolution
			for dt in edid.get_monitor_details():
				if dt[0].count("Detailed Timing"):
					t = dt[1]
					### FIXME Depth?
					for d in [8, 16, 24]:
						self.modes = self.modes + [ Mode( t['horizontal_active'], t['vertical_active'], d) ]
					
			### Add all other supported modes
			for t in edid.get_timings():
				### FIXME Depth?
				for d in [8, 16, 24]:
					self.modes = self.modes + [ Mode(t[0], t[1], d) ]		

		### We don't really have any info, so make a safe guess
		### (640x480 - 8bit) ?
		if len(self.modes) == 0:
			self.modes = self.modes + [ Mode(640, 480, 8) ]
			self.depth = [8]

	### Do any of our modes require modelines?
	def req_modelines(self):
		for m in self.modes:
			if not is_vesa_Mode(m):
				return 1
		return 0
	
	### Do any of our modes require Reduced Blanking?
	def req_reduced_blanking(self):
		for m in self.modes:
			if m.modeline.reduced_blanking:
				return 1
		return 0
	
	### Get Monitor Name
	def monitor_name(self):
		if self.edid != None:
			return self.edid.get_string_name()
		elif self.fb != None:
			return self.fb.fb_string()	
		else:
			return self.name

	### Generate monitor subsections
	def get_display_subsections(self, s):
		for d in self.depth:
			s.add('SubSection "Display"')
			s.add('\tDepth\t' + str(d))
			### Ensures that the default mode is the size of the virtual
			### window (people get confused if this isn't the case)
			s.add('\tVirtual\t' + str(self.modes[0].horiz) + " " + str(self.modes[0].vert))
			modes = '\tModes\t'
			for m in self.modes:
				if m.depth == d:
					modes = modes + m.res() + ' '
			s.add(modes)
			s.add('EndSubSection\n')
	
	### Update the default depth option
	def get_default_depth(self):
		if self.default_depth != None:
			return self.default_depth
		else:
			default_depth = 0
			if self.depth.count(15):
				default_depth = 15
			### Make the 32 bit depth actually 24 bits (packed)
			elif self.depth.count(32):
				default_depth = 24
			else:
				for d in self.depth:	
					for m in self.modes:
						if m.depth == d and d > default_depth:
							default_depth = d
			return default_depth
		
	### Generates the Monitor *and* Modes (if required)
	def get_section(self):
		s = Section()
		s.name("Monitor")
		s.add('Identifier\t"Monitor' + str(self.id) + '"')
		if self.req_modelines():
			s.add('UseModes\t"Modes' + str(self.id) + '"')
		s.add('ModelName\t"' + self.monitor_name() + '"')
		s.add('Option\t\t"DPMS"')
		if self.req_reduced_blanking():
			s.add('Option\t\t"ReducedBlanking"')
		### Ensure we have valid horiz / vert rates or guess
		if self.sync['h_min'] == 0 and self.sync['h_max'] == 0 and self.sync['v_min'] == 0 and self.sync['v_max'] == 0:
			### Find the Max X resolution
			max_x = 0
			for m in self.modes:
				if m.horiz > max_x:
					max_x = m.horiz

			### 640 or less
			if max_x <= 640:
				self.sync['h_min'] = 28
				self.sync['h_max'] = 33
				self.sync['v_min'] = 43
				self.sync['v_max'] = 72

			### 800 or less
			elif max_x <= 800:
				self.sync['h_min'] = 28
				self.sync['h_max'] = 50
				self.sync['v_min'] = 43
				self.sync['v_max'] = 75

			### 1024 or less
			elif max_x <= 1024:
				self.sync['h_min'] = 30
				self.sync['h_max'] = 70
				self.sync['v_min'] = 50
				self.sync['v_max'] = 160

			### 1152 or less
			elif max_x <= 1152:
				self.sync['h_min'] = 30
				self.sync['h_max'] = 100
				self.sync['v_min'] = 50
				self.sync['v_max'] = 160

			### All remaining resolutions
			else:
				self.sync['h_min'] = 30
				self.sync['h_max'] = 130
				self.sync['v_min'] = 50
				self.sync['v_max'] = 160

		### Apple Studio Display workaround
		elif self.sync['h_min'] == 0 and self.sync['h_max'] == 0 and self.sync['v_min'] != 0 and self.sync['v_max'] != 0:
			self.sync['h_min'] = 30
			self.sync['h_max'] = 130

		s.add('HorizSync\t' + str(self.sync['h_min']) + "-" + str(self.sync['h_max']))
		s.add('VertRefresh\t' + str(self.sync['v_min']) + "-" + str(self.sync['v_max']))
		config = s.get_section()
		### Add the Modelines if needed
		if self.req_modelines():
			printed_modes = []
			s = Section()
			s.name("Modes")
			s.add('Identifier\t"Modes' + str(self.id) + '"')
			for m in self.modes:
				if not printed_modes.count(m.res()):
					m.get_modeline(s)	
					printed_modes.append(m.res())
			config = config + s.get_section()
		### Generate the Screen section
		s = Section()
		s.name("Screen")
		s.add('Identifier\t"Screen' + str(self.id) + '"')
		s.add('Device\t\t"Card' + str(self.id) + '"')
		s.add('Monitor\t\t"Monitor' + str(self.id) + '"')
		s.add('DefaultDepth\t' + str(self.get_default_depth()) + '\n')
		if self.fb and self.fbdepth == 32:
			s.add('DefaultFbBpp\t32\n')
		self.get_display_subsections(s)
		config = config + s.get_section()
		return config	
